-- AddOn: VeteranBosses
-- Author: @poxxerom
-- Most of this code is a direct copy of Harven's Custom Map Pins addon.
-- To Do: 
-- 1) Add more dungeons :)
-- 2) Add Boss notifiers. Kinris DOT notifier ready but not yet implemented. Might include in separate addon.
-- 3) Add Boss timers, ie: Areldur AOE every xx seconds
-- 4) Ideas?
--
-- Description: This addon shows boss locations on the minimap, mouseover over the icons to show boss tactics.
--
-- Change Log:
-- Update 0.1: Original release
-- Update 0.2: Changed to match API version 100004 which came out with patch 1.1.2
-- Update 0.3: Removed readme.txt.
--             Added option to share boss tactics in chat
-- Update 0.4: Added Veteran Spindleclutch dungeon bosses and tactics
-- Update 0.5: Added Fungal Grotto dungeon bosses and tactics
-- Update 0.6: Added Wayrest Sewers dungeon bosses and tactics
-- Update 0.7: Added Darkshade Caverns dungeon bosses and tactics
-- Update 0.8: Added Elden Hollow dungeon bosses and tactics
-- Update 0.9: Added Aetherian Archive raid bosses and tactics
-- Update 1.0: Added Crypt of Hearts dungeon bosses and tactics. 
--             Removed colors from chat since 1.2.3 API disabled it.
--             Updated for API 100005
-- 
-- Known issues:
-- 1) When a quest circle is overlapping a boss, the popup says "nil" instead of "share tactics".

local VeteranBosses = {}
local SPversion = "0.9"
local ShowPins = false

function VeteranBosses:New()
	VeteranBosses:Initialize()
	return VeteranBosses
end

function VeteranBosses:CreatePin(cpin, key)
	local pin = self.pinManager:CreatePin(self.pinTypeId, key, cpin.x, cpin.y, 0)
	--save this function for later use in other functions
	if self.GetMapPinForControl == nil then
		self.GetMapPinForControl = pin.GetMapPinForControl
	end
	pin.VeteranBossesName = cpin.description
	pin.VeteranBossesTactics = cpin.tactics
	pin.VeteranBossesBugs = cpin.bugs
end

-- This can be done with a single function and loop, just haven't bothered with it 
function VeteranBosses:SharePart1(pTag)
	--CHAT_SYSTEM:StartTextEntry(string.sub(BossData.pins[pTag].tactics, 0))
	CHAT_SYSTEM:StartTextEntry(BossData.pins[pTag].tactics)
end

local function ShowPinMenu(pin, button)
	local pType, pTag = pin:GetPinTypeAndTag()
	--local tacticslength = string.len(BossData.pins[pTag].tactics)
	--local tacticsparts = math.modf(tacticslength/300)
	--local shareparts = tacticsparts + 1
	
	ClearMenu()
	-- This can be done with a single function and loop, just haven't bothered with it 
	--if shareparts > 0 then
		AddMenuItem("Share Tactics", function() VeteranBosses:SharePart1(pTag) end)
	--end
		
	ShowMenu(pin:GetControl(), 1)
	--dirty hack! because something calls ClearMenu after this function returns
	local backupfunc = ClearMenu
	ClearMenu = function(...) ClearMenu = backupfunc end
end

--This function is called by worldmap when changing map or after calling RefreshCustomPins
local function LayoutPins(pinManager)
	-- We only want to use the pins if we're in Veteran mode. 
	-- Alex needs to test this before release of 0.9
	if (InsideVeteranDungeon() or InsideRaidDungeon()) then
	--if ShowPins then
		--save pinManager for later use in other functions
		if VeteranBosses.pinManager == nil then
			VeteranBosses.pinManager = pinManager
		end
		-- Iterate through BossData.lua to get boss data information
		for k,v in pairs(BossData.pins) do
			if GetMapName() == v.map then
				--check if the pin really doesn't exists
				if pinManager:FindPin(VeteranBosses.pinTypeString, VeteranBosses.pinTypeId, k) == nil then
					VeteranBosses:CreatePin(v,k)	
				end
			end
		end
	end
end

function VeteranBosses:Initialize()
	local defaults = { pins = {}, point = CENTER, x = 0, y = 0,	relPoint = CENTER }
	self.pinTypeId = 0 -- this is important variable it will be set by ZO_WorldMap_AddCustomPin
	self.pinTypeString = "VeteranBossesType"
	self.requests = {}
	
	-- Creates the icon mouse-over tooltip
	local creator = {
		creator = function(pin)
			InformationTooltip:AddLine(pin.VeteranBossesName)
			InformationTooltip:AddLine(pin.VeteranBossesTactics)
			InformationTooltip:AddLine(pin.VeteranBossesBugs)
		end,
		tooltip = InformationTooltip,
	}

	local layout = {level=30,size=24,insetX=8,insetY=8,texture="VeteranBosses/GroupBoss.dds"}

	--define our custom pins
	ZO_WorldMap_AddCustomPin(self.pinTypeString, LayoutPins, nil, layout, creator)
	self.pinTypeId = _G[self.pinTypeString]
	ZO_WorldMap_SetCustomPinEnabled(self.pinTypeId, true)
	ZO_MapPin.PIN_CLICK_HANDLERS[1][self.pinTypeId] = {
		{
			name = function(pin) return tostring(pin.VeteranBossesName) end,
			show = function(...) return true end,
			callback = ShowPinMenu,
			duplicates = function(...) end,
		}
	}
end

function InsideVeteranDungeon()
	-- Returns TRUE if player is currently in a Veteran Dungeon
	local mycurrentsubzone = GetPlayerLocationName()
	if (IsUnitUsingVeteranDifficulty("player")) then
		if (mycurrentsubzone == "The Banished Cells") then return true end
		if (mycurrentsubzone == "Spindleclutch") then return true end
		if (mycurrentsubzone == "Fungal Grotto") then return true end
		if (mycurrentsubzone == "Wayrest Sewers") then return true end
		if (mycurrentsubzone == "Darkshade Caverns") then return true end
		if (mycurrentsubzone == "Elden Hollow") then return true end
		-- Daedric Realm is a portion of Elden Hollow
		if (mycurrentsubzone == "Daedric Realm") then return true end
		if (mycurrentsubzone == "Aetherian Archive") then return true end
		if (mycurrentsubzone == "Hel Ra Citadel") then return true end
		if (mycurrentsubzone == "Crypt of Hearts") then return true end
	end
	return false
end

function InsideRaidDungeon()
	-- Returns TRUE if player is currently in a Raid Dungeon
	local mycurrentsubzone = GetPlayerLocationName()
	if (mycurrentsubzone == "Aetherian Archive") then return true end
	if (mycurrentsubzone == "Hel Ra Citadel") then return true end
	return false
end

local function Intro()
		EVENT_MANAGER:UnregisterForEvent("VeteranBossesLoadHandler", EVENT_PLAYER_ACTIVATED)
		d("|c3A92FFVeteran|cFFD700Bosses|cFFFFFF version |c3A92FF" .. SPversion .. "|cFFFFFF loaded.")
		d("|c3A92FFVeteran|cFFD700Bosses|cFFFFFF currently supports \"The Banished Cells\"")
		d("|c3A92FFVeteran|cFFD700Bosses|cFFFFFF currently supports \"Spindleclutch\"")
		d("|c3A92FFVeteran|cFFD700Bosses|cFFFFFF currently supports \"Fungal Grotto\"")
		d("|c3A92FFVeteran|cFFD700Bosses|cFFFFFF currently supports \"Wayrest Sewers\"")
		d("|c3A92FFVeteran|cFFD700Bosses|cFFFFFF currently supports \"Darkshade Caverns\"")
		d("|c3A92FFVeteran|cFFD700Bosses|cFFFFFF currently supports \"Elden Hollow\"")
		d("|c3A92FFVeteran|cFFD700Bosses|cFFFFFF currently supports \"Aetherian Archive\"")
		d("|c3A92FFVeteran|cFFD700Bosses|cFFFFFF currently supports \"Crypt of Hearts\"")
end

local function VeteranBosses_Initialize(eventCode, addOnName)
	if addOnName ~= "VeteranBosses" then return end
	-- Work in progress. Might be better to listen for zone update event and set a global var than to use functions every time.
	-- EVENT_MANAGER:RegisterForEvent("VeteranBossesZoneUpdate", EVENT_ZONE_UPDATE, VeteranBosses_ZoneUpdate)
	VeteranBosses:New()
end

EVENT_MANAGER:RegisterForEvent("VeteranBossesLoadHandler", EVENT_ADD_ON_LOADED, VeteranBosses_Initialize)
EVENT_MANAGER:RegisterForEvent("VeteranBossesLoadHandler", EVENT_PLAYER_ACTIVATED, Intro)